#
###
# Application: Pycadia
# File:        gd_letters.py
# Description: pycadia vector letters data
# Copyright (c) 2002-2003   Andy Balcombe <http://www.anti-particle.com>
###
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
#

"""pycadia vector letters data"""

#letter vertex data
aVerts = [
    (-1.5,3), # A
    (-1.5,0),
    (-1.5,-3),
    (1.5,-3),
    (1.5,0),
    (1.5,3)]
bVerts = [
    (-1.5,3), # B
    (-1.5,-3),
    (0.75,-3),
    (1.5,-2.25),
    (1.5,-0.75),
    (0.75,0),
    (1.5,0.75),
    (1.5,2.25),
    (0.75,3),
    (-1.5,0)]
cVerts = [
    (-1.5,3), # C
    (-1.5,-3),
    (1.5,-3),
    (1.5,3)]
dVerts = [
    (-1.5,3), # D
    (-1.5,-3),
    (0.75,-3),
    (1.5,-2.25),
    (1.5,2.25),
    (0.75,3)]
eVerts = [
    (1.5,3),  # E
    (-1.5,3),
    (-1.5,0),
    (0.5,0),
    (-1.5,-3),
    (1.5,-3)]
fVerts = [
    (-1.5,3), # F
    (-1.5,0),
    (0.5,0),
    (-1.5,-3),
    (1.5,-3)]
gVerts = [
    (1.5,-3), # G
    (-1.5,-3),
    (-1.5,3),
    (1.5,3),
    (1.5,0),
    (0,0)]
hVerts = [
    (-1.5,3), # H
    (-1.5,-3),
    (1.5,-3),
    (1.5,3),
    (-1.5,0),
    (1.5,0)]
iVerts = [
    (0,-3),   # I
    (0,3),
    (-1,-3),
    (1,-3),
    (-1,3),
    (1,3)]
jVerts = [
    (-1.5,-3),  # J
    (1.5,-3),
    (0.25,-3),
    (0.25,3),
    (-1.5,3),
    (-1.5,2.25)]
kVerts = [
    (-1.5,3), # K
    (-1.5,-3),
    (1.5,-3),
    (-1.5,0),
    (1.5,3)]
lVerts = [
    (-1.5,-3),# L
    (-1.5,3),
    (1.5,3)]
mVerts = [
    (-1.5,3), # M
    (-1.5,-3),
    (0,0),
    (1.5,-3),
    (1.5,3)]
nVerts = [
    (-1.5,3), # N
    (-1.5,-3),
    (1.5,-3),
    (1.5,3)]
oVerts = [
    (-1.5,3), # O
    (-1.5,-3),
    (1.5,-3),
    (1.5,3)]
pVerts = [
    (-1.5,3), # P
    (-1.5,-3),
    (1.5,-3),
    (-1.5,0),
    (1.5,0)]
qVerts = [
    (-0.75,3),
    (-1.5,2.25), # Q
    (-1.5,-2.25),
    (-0.75,-3),
    (0.75,-3),
    (1.5,-2.25),
    (1.5,2.25),
    (0.75,3),
    (1.5,3),
    (0.75,2.25)]
rVerts = [
    (-1.5,3), # R
    (-1.5,0),
    (-1.5,-3),
    (1.5,-3),
    (1.5,0),
    (1.5,3)]
sVerts = [
    (-1.5,3), # S
    (1.5,3),
    (1.5,0),
    (-1.5,0),
    (-1.5,-3),
    (1.5,-3)]
tVerts = [
    (0,-3),   # T
    (0,3),
    (-1.5,-3),
    (1.5,-3)]
uVerts = [
    (-1.5,3), # U
    (-1.5,-3),
    (1.5,-3),
    (1.5,3)]
vVerts = [
    (-1.5,-3),# V
    (-1.5,0),
    (0,3),
    (1.5,0),
    (1.5,-3)]
wVerts = [
    (-1.5,-3),# W
    (-1.5,0),
    (-0.75,3),
    (0,0),
    (0.75,3),
    (1.5,0),
    (1.5,-3)]
xVerts = [
    (-1.5,3), # X
    (-1.5,-3),
    (1.5,-3),
    (1.5,3)]
yVerts = [
    (-1.5,-3),# Y
    (0,0),
    (1.5,-3),
    (0,3)]
zVerts = [
    (-1.5,3), # Z
    (-1.5,-3),
    (1.5,-3),
    (1.5,3)]

#letter line data
aLines = [
    (0,2),    # A
    (2,3),
    (3,5),
    (1,4)]
bLines = [
    (0,1),    # B
    (1,2),
    (2,3),
    (3,4),
    (4,5),
    (5,6),
    (6,7),
    (7,8),
    (8,0),
    (9,5)]
cLines = [
    (0,1),    # C
    (1,2),
    (3,0)]
dLines = [
    (0,1),    # D
    (1,2),
    (2,3),
    (3,4),
    (4,5),
    (5,0)]
eLines = [
    (0,1),    # E
    (1,4),
    (4,5),
    (2,3)]
fLines = [
    (0,3),    # F
    (3,4),
    (1,2)]
gLines = [
    (0,1),    # G
    (1,2),
    (2,3),
    (3,4),
    (4,5)]
hLines = [
    (0,1),    # H
    (2,3),
    (4,5)]
iLines = [
    (0,1),    # I
    (2,3),
    (4,5)]
jLines = [
    (0,1),    # J
    (2,3),
    (3,4),
    (4,5)]
kLines = [
    (0,1),    # K
    (3,2),
    (3,4)]
lLines = [
    (0,1),    # L
    (1,2)]
mLines = [
    (0,1),    # M
    (1,2),
    (2,3),
    (3,4)]
nLines = [
    (0,1),    # N
    (1,3),
    (3,2)]
oLines = [
    (0,1),    # O
    (1,2),
    (2,3),
    (3,0)]
pLines = [
    (0,1),    # P
    (1,2),
    (2,4),
    (3,4)]
qLines = [
    (0,1),    # Q
    (1,2),
    (2,3),
    (3,4),
    (4,5),
    (5,6),
    (6,7),
    (7,0),
    (8,9)]
rLines = [
    (0,2),    # R
    (2,3),
    (3,4),
    (4,1),
    (1,5)]
sLines = [
    (0,1),    # S
    (1,2),
    (2,3),
    (3,4),
    (4,5)]
tLines = [
    (0,1),    # T
    (2,3)]
uLines = [
    (0,1),    # U
    (2,3),
    (3,0)]
vLines = [
    (0,1),    # V
    (1,2),
    (2,3),
    (3,4)]
wLines = [
    (0,1),    # W
    (1,2),
    (2,3),
    (3,4),
    (4,5),
    (5,6)]
xLines = [
    (0,2),    # X
    (1,3)]
yLines = [
    (0,1),    # Y
    (1,2),
    (1,3)]
zLines = [
    (1,2),    # Z
    (2,0),
    (0,3)]
        
#initial rect sizes (size of sprite)
letterRectSize = (5, 8)

